@@ -23,6 +23,8 @@ def parse_args(args, log):
 def parse_config(path, log):
     """Parse yaml collection configuration."""
     configuration = load_config(path)
+    if 'name' in configuration:
+        configuration['name'] = str(configuration['name'])
     validate_config(configuration, log)
     return configuration
 
