@@ -5,13 +5,16 @@
 import concurrent.futures
 import os
 import uuid
+from urllib.parse import urlparse
 
+import azure.storage.blob
 from colorama import Fore, Style
 from tqdm import tqdm
 
+
 from . import (
-    config,
     azure_blobs,
+    config,
     http,
 )
 
@@ -25,6 +28,7 @@ MIMES = {
     ".tif": "image/tiff",
     ".tiff": "image/tiff",
     ".dcm": "application/dicom",
+    # TODO add WSI mime types?
 }
 
 BLACKLIST = (
@@ -57,7 +61,7 @@ def get(log, session, args):
 
 def _get_chunk_upload_futures(
     executor, paths, session, create_url,
-    complete_url, log, workload_size, offset, mime
+    complete_url, log, workload_size, offset, mime, use_azure_client
 ):
     """Return executable tasks with image uploads in batches.
 
@@ -88,7 +92,8 @@ def _get_chunk_upload_futures(
                 complete_url,
                 log,
                 workload_info,
-                mime
+                mime,
+                use_azure_client,
             ))
             temp = []
 
@@ -101,7 +106,7 @@ def _finish_replace_empty_imageset(session, replace_empty_url):
     http.post_json(session, replace_empty_url, {})
 
 
-def _upload_image_chunked(paths, session, create_url, complete_url, log, workload_info, mime):  # noqa: E501
+def _upload_image_chunked(paths, session, create_url, complete_url, log, workload_info, mime, use_azure_client=False):  # noqa: E501
     results = []
 
     # get all signed urls at once
@@ -113,6 +118,7 @@ def _upload_image_chunked(paths, session, create_url, complete_url, log, workloa
         return
 
     index = 0
+
     for fpath in paths:
         try:
             file_name = os.path.basename(fpath)
@@ -138,12 +144,28 @@ def _upload_image_chunked(paths, session, create_url, complete_url, log, workloa
             try:
                 # Post file to storage location
                 url = signed_urls[blob_id]
-                # google based signed urls come as a relative path
-                if url.startswith("/"):
-                    url = 'https://storage.googleapis.com{}'.format(url)
-                http.put_file(session, url, f, file_mime)
-                # pop the info into a temp array, upload only once later
-                results.append(info["image"])
+
+                if use_azure_client:
+                    url_object = urlparse(url)
+                    # get SAS token from url
+                    sas_token = url_object.query
+                    account_url = url_object.scheme + '://' + url_object.netloc
+                    container_name = url_object.path.split('/')[1]
+
+                    # upload blob using client
+                    blob_client = azure.storage.blob.ContainerClient(account_url, container_name, credential=sas_token)
+                    blob_client.upload_blob(blob_id, f)
+
+                    results.append(info["image"])
+                else:
+                    # TODO fetch project storage location to decide this
+                    is_gcp_storage = url.startswith("/")
+                    if is_gcp_storage:
+                        url = 'https://storage.googleapis.com{}'.format(url)
+                    http.put_file(session, url, f, file_mime)
+                    # pop the info into a temp array, upload only once later
+                    results.append(info["image"])
+
             except Exception as ex:
                 log.error("File upload failed: {}".format(ex))
 
@@ -208,6 +230,8 @@ def _update_file_imageset(log, session, configuration):
     if workload_size != 1:
         log.warn("The progress bar may have reduced accuracy when uploading larger imagesets.")  # noqa: E501
 
+    use_azure_client = configuration.get('use_wsi', False)
+
     with concurrent.futures.ThreadPoolExecutor(http.CONCURRENCY) as executor:
         futures = _get_chunk_upload_futures(
             executor,
@@ -218,7 +242,8 @@ def _update_file_imageset(log, session, configuration):
             log,
             workload_size,
             add_offset,
-            mime_type
+            mime_type,
+            use_azure_client,
         )
         kwargs = {
             'total': len(futures),
@@ -255,6 +280,14 @@ def _update_join_dataset(
     collection_response = http.get(session, collection_url)
     collection = collection_response['collection']
 
+    if 'source_name' in configuration:
+        source_name = configuration['source_name']
+        for source in collection["image_sources"]:
+            if source['name'] == source_name:
+                ims_ds_join_id = source["imageset_dataset_join_id"]
+    else:
+        ims_ds_join_id = collection["imageset_dataset_join_id"]
+
     # update the join dataset
     join_data = {
         'name': 'join dataset',
@@ -270,7 +303,7 @@ def _update_join_dataset(
     }
     imageset_dataset_join_url = "{}datasets/{}".format(
         http.get_api_url(configuration["url"], configuration["project"]),
-        collection["imageset_dataset_join_id"]
+        ims_ds_join_id
     )
     log.debug('PUT: {}'.format(imageset_dataset_join_url))
     http.put_json(session, imageset_dataset_join_url, join_data)
